import java.util.*;
import java.awt.*;
import java.io.*;

import sv.kernel.*;
import sv.ui.SVUIC;

/** 
 * FrameManger.java
 * Manage the visualization frames
 * @version 1.0   10/1/97 
 * @author Byeongseob Ki
 */

public class FrameManager {

  private static svserver svServer;

  // GFrame vectors
  private static Vector frame1d_vec;  
  private static Vector frame2d_vec;  
  private static Vector frame3d_vec;  

  // toggle for global command
  private boolean isGlobalCommand = false;   

  public FrameManager(svserver svServer) {
     this.svServer = svServer;

     frame1d_vec = new Vector();
     frame2d_vec = new Vector();
     frame3d_vec = new Vector();
  }


  /**
   * global command checking
   */
  public void globalCheck(boolean b) {
      if (b)
        svServer.cmanager.broadcastCommand(null, -2, null);

      if (isGlobalCommand) {
         isGlobalCommand = false;
         svServer.message("Command broadcast set false ...", false);
      } else {
         isGlobalCommand = true;
         svServer.message("Command broadcast set true ...", false);
      }
  }

  /**
   * set drawingmode for each visualization frame
   */
  public void setDrawingMode(boolean b) {
     for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
          GFPlane gframe = (GFPlane)e.nextElement();
          gframe.canvas.drawingmode = b;
     }
     for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
          GFrame2D gframe = (GFrame2D)e.nextElement();
          gframe.canvas.drawingmode = b;
     }
     for (Enumeration e=frame3d_vec.elements(); e.hasMoreElements();) {
          GFrame3D gframe = (GFrame3D)e.nextElement();
          gframe.canvas.drawingmode = b;
     }
  }


  /** 
   * register data set to this system
   * @param data data set to visualize
   * @param id expected id
   * @param c configurations
   */
  
  public long register(DataSet dataset, long id, Configs c) {
     long fid = -1;
     int dim = dataset.getRank();

     switch(dim) {
        case DataSet.ONED:
           fid = register_1d(dataset.getTitle(), dataset, id, (CPlane)c);
           break;
        case DataSet.TWOD:
           fid = register_2d(dataset.getTitle(), dataset, id, (Config2D)c);
           break;
        case DataSet.THREED:
           fid = register_3d(dataset.getTitle(), dataset, id, (Config3D)c);
           break;
        default:
           System.out.println("Unknown data set");
     }
     return fid;
  }

  /** 
   * received data set from the clients
   * @param data received data set
   * @param c configurations
   * @param id frame id
   * @param b if true, broadcast data set
   */
  public void receivedDataSet(DataSet data, Configs c, 
                                              long id, boolean b) {
     int dim = data.getRank();

     switch(dim) {
        case DataSet.ONED:
           received_1d(data, (CPlane)c, id, b);
           break;

        case DataSet.TWOD:
           received_2d(data, (Config2D)c, id, b);
           break;

        case DataSet.THREED:
           received_3d(data, (Config3D)c, id, b);
           break;

        default:
           System.out.println("Unknown data set");
     }
  }   


  /**
   * action for 1d frame window
   * @param gf1d source GFPlane
   * @param id command id
   * @param args arguments
   * @param b if true, broadcast
   */
  public void action1d(GFPlane gf1d, int id, String[] args, boolean b) {
    switch(id) {
       case CommandSet.QUIT: 
         if (isGlobalCommand) {
           while (!frame1d_vec.isEmpty()) {
              GFPlane gframe = (GFPlane)frame1d_vec.lastElement();
              gframe.killFrame();
           }
           svServer.message("Killed all 1d windows.", false);

         } else 
           gf1d.killFrame();

         break;

       case CommandSet.ANIMATE:
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.doAnimate();
	   }
         } else 
           gf1d.doAnimate();

         break;

       case CommandSet.STOP:
          int time = Integer.valueOf(args[0]).intValue();
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.doStop();
              if (time >= gframe.noOfTime)
                time = gframe.noOfTime-1;
              gframe.currentTime = time;
	   }
	  } else {
            gf1d.doStop();
            if (time >= gf1d.noOfTime)
              time = gf1d.noOfTime-1;
            gf1d.currentTime = time;
          }

         break;

       case CommandSet.NEXT: 
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.next(true);
	   }
         } else 
           gf1d.next(true);

         break;

       case CommandSet.PREVIOUS: 
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.previous();
	   }
         } else 
           gf1d.previous();

         break;

       case CSPlane.LEGENDMOVE:
       case CSPlane.LABELMOVE:
       case CSPlane.TICKERCHANGE: 
       case CommandSet.BACKCOLOR: 
       case CSPlane.RESIZEPDA: 
       case CSPlane.BASELINEMOVE: 
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.doAction_2(id, args);
	   }
         } else 
           gf1d.doAction_2(id, args);

         break;

       case CSPlane.CONNECTPOINT:
       case CSPlane.DISPLAYLBL:
       case CSPlane.TICKMARK:
       case CSPlane.GMINMAX:
       case CSPlane.AMR:
       case CSPlane.COLORBYINFO:
       case CSPlane.COLORBYSET:
       case CSPlane.DISPLAYLGD:
       case CSPlane.DISPLAYTIME:
       case CSPlane.FILTEROVERLAY:
       case CSPlane.FILTERDISCARD:
       case CSPlane.BASELINE:
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.doAction_1(id, args[0]);
	   }
         } else 
           gf1d.doAction_1(id, args[0]);

         break;

       case CSPlane.SPLIT: 
         gf1d.doSplit();
         break;

       case CSPlane.LABELCHANGE: 
         gf1d.doLabelChange(args);
         break;

       case CSPlane.LEGENDCHANGE: 
         ((GFPOneD)gf1d).doLegendChange(args);
         break;

       case CSPlane.RENAMETITLE: 
         for (int i=0; i<gf1d.dataSetVector.size(); i++) {
            DataSet data = (DataSet)gf1d.dataSetVector.elementAt(i);

            if (args[0].equals(data.getTitle())) {
               data.setTitle(args[1]);

               String str;
               if (gf1d.dataSetVector.size() == 1) 
                  str = data.getTitle();
               else
                  str = data.getTitle() + "...";

               gf1d.setFrameTitle(str);
               break;
            }
         }
         gf1d.canvas.repaint();
         gf1d.monitor.display("DataSet names are  changed ...", false);
         break;

       case CSPlane.ZOOMOUT:
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.canvas.zoomOut();
              gframe.monitor.display("Zoomed out ...", false);
	   }
         } else {
           gf1d.canvas.zoomOut();
           gf1d.monitor.display("Zoomed out ...", false);
         }
         break;

       case CSPlane.RESET:
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.canvas.resetZooming();
              gframe.monitor.display("Reseted all zooming factors ...", false);
	   }
         } else {
           gf1d.canvas.resetZooming();
           gf1d.monitor.display("Reseted all zooming factors ...", false);
         }

         break;

       case CSPlane.GOTO:
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.doGotoAction(args[0]);
	   }
         } else 
           gf1d.doGotoAction(args[0]);

         break;

       case CSPlane.BUFFERSIZE:
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.doBufferAction(args[0]);
	   }
         } else 
           gf1d.doBufferAction(args[0]);

         break;

       case CSPlane.ZOOMING:
         gf1d.canvas.colZooms(args);
         break;

       case CSPlane.ASPEEDCHANGE:
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.doAnimationSpeedChange(Integer.valueOf(args[0]).intValue());
	   }
         } else 
           gf1d.doAnimationSpeedChange(Integer.valueOf(args[0]).intValue());

         break;


       case CommandSet.COLORMAPCHANGE:
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.setColorMap(args);
	   }
         } else
           gf1d.setColorMap(args);

         break;


       case CommandSet.WCUT:
         gf1d.canvas.doWCut(Integer.valueOf(args[0]).intValue());
         break;

       case CommandSet.WCOPY:
         gf1d.canvas.doWCopy(Integer.valueOf(args[0]).intValue());
         break;

       case CommandSet.WPASTE:
         gf1d.canvas.doWPaste();
         break;

       case CommandSet.WCREATE:
         gf1d.canvas.doWCreate(args[0]);
         break;

       case CommandSet.WADDPOINT:
         gf1d.canvas.doWAddPoint(Integer.valueOf(args[0]).intValue(),
                                 Integer.valueOf(args[1]).intValue(),
                                 Integer.valueOf(args[2]).intValue());

         break;

       case CommandSet.WADDPOINTS:
         gf1d.canvas.doWAddPoints(args);
         break;

       case CommandSet.WHSCREATE:
         gf1d.canvas.doWHSCreate(Integer.valueOf(args[0]).intValue());
         break;

       case CommandSet.WHSMOVE:
         gf1d.canvas.doWHSMove(Integer.valueOf(args[0]).intValue(),
                               Integer.valueOf(args[1]).intValue());
         break;

       case CommandSet.WFILL:
         gf1d.canvas.doWFill(Integer.valueOf(args[0]).intValue());
         break;

       case CommandSet.WLINECOLOR:
         gf1d.canvas.doWLineColor(Integer.valueOf(args[0]).intValue(),
                                  Integer.valueOf(args[1]).intValue());
         break;

       case CommandSet.WFILLCOLOR:
         gf1d.canvas.doWFillColor(Integer.valueOf(args[0]).intValue(),
                                  Integer.valueOf(args[1]).intValue());
         break;

       case CommandSet.WTEXTCOLOR:
         gf1d.canvas.doWTextColor(Integer.valueOf(args[0]).intValue(),
                                  Integer.valueOf(args[1]).intValue());
         break;

       case CommandSet.WTEXTFONT:
         gf1d.canvas.doWTextFont(Integer.valueOf(args[0]).intValue(),
                                 args[1],
                                 Integer.valueOf(args[2]).intValue(),
                                 Integer.valueOf(args[3]).intValue());
         break;

       case CommandSet.WPEN:
         gf1d.canvas.doWPen(Integer.valueOf(args[0]).intValue(),
                                 Integer.valueOf(args[1]).intValue(),
                                 Integer.valueOf(args[2]).intValue(),
                                 Integer.valueOf(args[3]).intValue());
         break;


    }
    if (b)
      svServer.cmanager.broadcastCommand(gf1d, id, args);
  }


  /**
   * action for 2d frame window
   * @param gf2d source GFrame2D
   * @param id command id
   * @param args arguments
   * @param b if true, broadcast
   */
  public void action2d(GFrame2D gf2d, int id, String[] args, boolean b) {

    switch(id) {
       case CommandSet2D.QUIT: 
         if (isGlobalCommand) {
           while (!frame2d_vec.isEmpty()) {
              GFrame2D gframe = (GFrame2D)frame2d_vec.lastElement();
              gframe.killFrame();
           }
           svServer.message("Killed all 2d windows.", false);

         } else 
           gf2d.killFrame();

         break;

       case CommandSet2D.ANIMATE:
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.doAnimate();
	   }
         } else 
           gf2d.doAnimate();

         break;

       case CommandSet2D.STOP:
          int time = Integer.valueOf(args[0]).intValue();
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.doStop();
              if (time >= gframe.noOfTime)
                 time = gframe.noOfTime-1;
              gframe.currentTime = time;
	   }
          } else {
           gf2d.doStop();
           if (time >= gf2d.noOfTime)
              time = gf2d.noOfTime-1;
           gf2d.currentTime = time;
         }

         break;

       case CommandSet2D.NEXT: 
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.next(true);
	   }
         } else 
           gf2d.next(true);

         break;

       case CommandSet2D.PREVIOUS: 
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.previous();
	   }
         } else 
           gf2d.previous();

         break;

       case CommandSet2D.ASPEEDCHANGE:
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.doAnimationSpeedChange(Integer.valueOf(args[0]).intValue());
	   }
         } else 
           gf2d.doAnimationSpeedChange(Integer.valueOf(args[0]).intValue());

         break;


       case CommandSet2D.SPLIT: 
         gf2d.doSplit();
         break;

       case CommandSet2D.RESET:
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.canvas.reset();
	   }
         } else 
           gf2d.canvas.reset();

         break;

       case CommandSet2D.GOTO:
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.doGotoAction(args[0]);
	   }
         } else 
           gf2d.doGotoAction(args[0]);

         break;

       case CommandSet.BUFFERSIZE:
         if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.doBufferAction(args[0]);
	   }
         } else 
           gf2d.doBufferAction(args[0]);

         break;


       case CommandSet2D.LABELCHANGE:
         gf2d.doLabelChange(args);
         break;

       case CommandSet2D.BOUNDBOX:
       case CommandSet2D.XYMESH:
       case CommandSet2D.FACEGRID:
       case CommandSet2D.VECTOR:
       case CommandSet2D.XYTICK:
       case CommandSet2D.ZTICK:
       case CommandSet2D.GMINMAX:
       case CommandSet2D.REGENERATE:
       case CommandSet2D.FILTEROVERLAY:
       case CommandSet2D.FILTERDISCARD:
       case CommandSet2D.DISPLAYTIME:
       case CommandSet2D.XSCALE:
       case CommandSet2D.YSCALE:
       case CommandSet2D.ZSCALE:
       case CommandSet2D.ALLSCALE:
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.doAction_1(id, args[0]);
	   }
         } else 
           gf2d.doAction_1(id, args[0]);

         break;

       case CommandSet2D.WIRE:
       case CommandSet2D.WIRECOLOR:
       case CommandSet2D.HIDDEN:
       case CommandSet2D.COLOR:
       case CommandSet2D.GRAY:
       case CommandSet2D.POINTPLOT:
       case CommandSet2D.SURFACE:
       case CommandSet2D.CONTOUR:
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.doAction_2(id);
	   }
         } else 
           gf2d.doAction_2(id);

         break;

       case CommandSet2D.BACKCOLOR:
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.doBackColor(args);
	   }
         } else 
           gf2d.doBackColor(args);

         break;

       case CommandSet2D.RENAMETITLE: 
         for (int i=0; i<gf2d.dataSetVector.size(); i++) {
            DataSet data = (DataSet)gf2d.dataSetVector.elementAt(i);

            if (args[0].equals(data.getTitle())) {
               data.setTitle(args[1]);

               String str;
               if (gf2d.dataSetVector.size() == 1) 
                  str = data.getTitle();
               else
                  str = data.getTitle() + "...";

               gf2d.setFrameTitle(str);
               break;
            }
         }
         gf2d.canvas.repaint();
         gf2d.monitor.display("DataSet names are  changed ...", false);
         break;

       case CommandSet2D.CONTOURMODIFY:
         gf2d.canvas.modifyContour(Integer.valueOf(args[0]).intValue());
         break;

       case CommandSet2D.POSITIONCHANGE:
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.canvas.setPosition(Float.valueOf(args[0]).floatValue(),
                                        Integer.valueOf(args[1]).intValue(),
                                        Integer.valueOf(args[2]).intValue(),
                                        Float.valueOf(args[3]).floatValue(),
                                        Float.valueOf(args[4]).floatValue(),
                                        Float.valueOf(args[5]).floatValue());
	   }
         } else
           gf2d.canvas.setPosition(Float.valueOf(args[0]).floatValue(),
                                        Integer.valueOf(args[1]).intValue(),
                                        Integer.valueOf(args[2]).intValue(),
                                        Float.valueOf(args[3]).floatValue(),
                                        Float.valueOf(args[4]).floatValue(),
                                        Float.valueOf(args[5]).floatValue());

         break;

       case CommandSet.COLORMAPCHANGE:
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.setColorMap(args);
	   }
         } else
           gf2d.setColorMap(args);

         break;

    }

    if (b)
      svServer.cmanager.broadcastCommand(gf2d, id, args);
  }

  /**
   * action for vector frame window
   * @param gf3d source GFrameVec
   * @param id command id
   * @param args arguments
   * @param b if true, broadcast
   */
  public void action3d(GFrame3D gf3d, int id, String[] args, boolean b) {
     switch(id) {
       case CommandSet3D.QUIT: 
         if (isGlobalCommand) {
           while (!frame3d_vec.isEmpty()) {
              GFrame3D gframe = (GFrame3D)frame3d_vec.lastElement();
              gframe.killFrame();
           }
           svServer.message("Killed all 3d windows.", false);

         } else 
           gf3d.killFrame();

         break;

       case CommandSet3D.ANIMATE:
          if (isGlobalCommand) {
           for (Enumeration e=frame3d_vec.elements(); e.hasMoreElements();) {
              GFrame3D gframe = (GFrame3D)e.nextElement();
              gframe.doAnimate();
	   }
         } else 
           gf3d.doAnimate();

         break;

       case CommandSet3D.STOP:
          int time = Integer.valueOf(args[0]).intValue();
          if (isGlobalCommand) {
           for (Enumeration e=frame3d_vec.elements(); e.hasMoreElements();) {
              GFrame3D gframe = (GFrame3D)e.nextElement();
              gframe.doStop();
              if (time >= gframe.noOfTime)
                time = gframe.noOfTime-1;
              gframe.currentTime = time;
	   }
	  } else {
            gf3d.doStop();
            if (time >= gf3d.noOfTime)
              time = gf3d.noOfTime-1;
            gf3d.currentTime = time;
          }

         break;

       case CommandSet3D.NEXT: 
          if (isGlobalCommand) {
           for (Enumeration e=frame3d_vec.elements(); e.hasMoreElements();) {
              GFrame3D gframe = (GFrame3D)e.nextElement();
              gframe.next(true);
	   }
         } else 
           gf3d.next(true);

         break;

       case CommandSet3D.PREVIOUS: 
          if (isGlobalCommand) {
           for (Enumeration e=frame3d_vec.elements(); e.hasMoreElements();) {
              GFrame3D gframe = (GFrame3D)e.nextElement();
              gframe.previous();
	   }
         } else 
           gf3d.previous();

         break;

       case CommandSet3D.SPLIT: 
         gf3d.doSplit();
         break;

       case CommandSet3D.BOUNDBOX:
       case CommandSet3D.XGRID:
       case CommandSet3D.YGRID:
       case CommandSet3D.ZGRID:
           if (isGlobalCommand) {
           for (Enumeration e=frame3d_vec.elements(); e.hasMoreElements();) {
              GFrame3D gframe = (GFrame3D)e.nextElement();
              gframe.doAction_2(id, args[0]);
	   }
         } else 
           gf3d.doAction_2(id, args[0]);

         break;
       case CommandSet3D.GMINMAX:
       case CommandSet3D.WIRE:
       case CommandSet3D.FILL:
       case CommandSet3D.SHADING:
          if (isGlobalCommand) {
           for (Enumeration e=frame3d_vec.elements(); e.hasMoreElements();) {
              GFrame3D gframe = (GFrame3D)e.nextElement();
              gframe.doAction_1(id, args[0]);
	   }
         } else 
           gf3d.doAction_1(id, args[0]);

         break;

       case CommandSet3D.GOTO:
          if (isGlobalCommand) {
           for (Enumeration e=frame3d_vec.elements(); e.hasMoreElements();) {
              GFrame3D gframe = (GFrame3D)e.nextElement();
              gframe.doGotoAction(args[0]);
	   }
         } else 
           gf3d.doGotoAction(args[0]);

         break;

       case CommandSet.COLORMAPCHANGE:
          if (isGlobalCommand) {
           for (Enumeration e=frame3d_vec.elements(); e.hasMoreElements();) {
              GFrame3D gframe = (GFrame3D)e.nextElement();
              gframe.setColorMap(args);
	   }
         } else
           gf3d.setColorMap(args);
         break;

       case CommandSet3D.POSITIONCHANGE:
          if (isGlobalCommand) {
           for (Enumeration e=frame3d_vec.elements(); e.hasMoreElements();) {
              GFrame3D gframe = (GFrame3D)e.nextElement();
              gframe.canvas.setPosition(Float.valueOf(args[0]).floatValue(),
                                        Float.valueOf(args[1]).floatValue());
	   }
         } else
           gf3d.canvas.setPosition(Float.valueOf(args[0]).floatValue(),
                                        Float.valueOf(args[1]).floatValue());

         break;

     } 

     if (b)
       svServer.cmanager.broadcastCommand(gf3d, id, args);
  }


  /**
   * action for global variables
   * @param id command id
   * @param args arguments
   * @param b if true, broadcast
   */
  public void actionGlobal(int id, String[] args, boolean b) {
    switch (id) {
       case -1:
         killAllFrames(false);
         break;

       case -2:
         globalCheck(false);
         break;

       case CommandSet.WLINECOLOR:
         svServer.shapes.setLineColor(new Color(Integer.valueOf(args[0]).intValue()));

         if (b)
           svServer.cmanager.broadcastCommand(null, id, args);
         break;

       case CommandSet.WFILLCOLOR:
         svServer.shapes.setFillColor(new Color(Integer.valueOf(args[0]).intValue()));

         if (b)
           svServer.cmanager.broadcastCommand(null, id, args);
         break;

       case CommandSet.WTEXTCOLOR:
         svServer.shapes.setTextColor(new Color(Integer.valueOf(args[0]).intValue()));

         if (b)
           svServer.cmanager.broadcastCommand(null, id, args);
         break;

       case CommandSet.WTEXTFONT:
         svServer.shapes.setTextFont(new Font(args[0], 
                         Integer.valueOf(args[1]).intValue(),
                         Integer.valueOf(args[2]).intValue()));

         if (b)
           svServer.cmanager.broadcastCommand(null, id, args);
         break;

       case CommandSet.WPEN:
         svServer.shapes.setLineType(Integer.valueOf(args[0]).intValue());
         svServer.shapes.setLineWidth(Integer.valueOf(args[1]).intValue());
         svServer.shapes.setDashSize(Integer.valueOf(args[2]).intValue());

         if (b)
           svServer.cmanager.broadcastCommand(null, id, args);
         break;

    }

  }


  /**
   * action for filter
   * @param id frame id
   * @param dim frame dimension
   * @param Filter filter to be performed
   * @param b if true, broadcast
   */
  public void actionFilter(long id, int dim, String[] rids, 
                                Filter filter, String input, boolean b) {
 
    boolean bycodes = false;
    byte[] result = null;
    String fname = null;
    int size;

    try {
       if (b) {
          Properties dprop = System.getProperties();
          String file_seperator = dprop.getProperty("file.separator");
          String java_path = dprop.getProperty("java.class.path");
          String path_separator = dprop.getProperty("path.separator");
          StringTokenizer st = new StringTokenizer(java_path, path_separator);
          String classname = filter.getClass().getName();
          StringTokenizer st2 = new StringTokenizer(classname, ".");

          String cname = "";
          size = st2.countTokens(); 
          for (int i=0; i<size-1; i++) 
             cname += st2.nextToken() + file_seperator;

          fname = st2.nextToken();
          cname += (fname + ".class");

          String path;
          size = st.countTokens();
          for (int i=0; i<size; i++) {
             path = st.nextToken();

             if (path.indexOf(".jar") > 0) continue;

             File target = new File(path, cname);
             if (target == null) continue;
             if (!target.exists()) continue;

             else {
               bycodes = true;
               DataInputStream in = new DataInputStream(new FileInputStream(target));
               int sz = in.available();
               result = new byte[sz];
    	       in.readFully(result, 0, sz); 
               break;
             }
          }
       }
    } catch (Exception E) {
       bycodes = false; 
    }

   
    boolean broadcasting = bycodes ? false : true;
    if (broadcasting && !b) broadcasting = false;

    if (dim == DataSet.ONED) {

       if (isGlobalCommand) {
         size = frame1d_vec.size();
         for (int i=0; i<size; i++) {
            GFPlane gframe = (GFPlane)frame1d_vec.elementAt(i);
            if (gframe.stealthMode) { broadcasting = b = false; }
            gframe.doTypeOne(filter, input, broadcasting);
	 }
       } else {
         GFPlane gframe = (GFPlane)getGFrame(id);
         if (gframe.stealthMode) { broadcasting = b = false; }
         gframe.doTypeOne(filter, input, broadcasting);
       }

       if (b && bycodes) {
          String[] idArray = new String[frame1d_vec.size() * 2];
          int index = 0;
          // window frame id adjustment
          for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
             GFPlane gframe = (GFPlane)e.nextElement();
             idArray[index] = gframe.getFrameTitle(); index++;
             idArray[index] = String.valueOf(gframe.getID()); index++;
          }
          svServer.cmanager.broadcastFilter(id, dim, fname, 
                                                  result, input, idArray);

       } else if (!b) {
          if (rids == null) return;
          for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
             GFPlane gframe = (GFPlane)e.nextElement();

             for (int i=0; i<rids.length; i=i+2) {
	       if (rids[i].equals(gframe.getFrameTitle())) {
                  gframe.setID(Long.valueOf(rids[i+1]).longValue());
                  break;
               }
             }
          }
       }
       

    } else if (dim == DataSet.TWOD) {
       if (filter.getOutDimension() == DataSet.ONED) broadcasting = false;

       if (isGlobalCommand) {
         size = frame2d_vec.size();
         for (int i=0; i<size; i++) {
            GFrame2D gframe = (GFrame2D)frame2d_vec.elementAt(i);
            if (gframe.stealthMode) { broadcasting = b = false; }
            gframe.doTypeOne(filter, input, broadcasting);
	 }
       } else {
         GFrame2D gframe = (GFrame2D)getGFrame(id);
         if (gframe.stealthMode) { broadcasting = b = false; }
         gframe.doTypeOne(filter, input, broadcasting);
       }

       if (b && bycodes) {
          String[] idArray = new String[frame2d_vec.size() * 2];
          int index = 0;
          // window frame id adjustment
          for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
             GFrame2D gframe = (GFrame2D)e.nextElement();
             idArray[index] = gframe.getFrameTitle(); index++;
             idArray[index] = String.valueOf(gframe.getID()); index++;
          }
          svServer.cmanager.broadcastFilter(id, dim, fname, 
                                                  result, input, idArray);

       } else if (!b) {
          if (rids == null) return;
          for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
             GFrame2D gframe = (GFrame2D)e.nextElement();

             for (int i=0; i<rids.length; i=i+2) {
	       if (rids[i].equals(gframe.getFrameTitle())) {
                  gframe.setID(Long.valueOf(rids[i+1]).longValue());
                  break;
               }
             }
          }
       }
    } 
  }
    

  /**
   * remove GFrame from the server
   * @param id DataSet id
   * @param dim dimension of the Graph Frame(1, 2, 3)
   */
  public void removeNotify(long id, int dim) {
    if (dim == DataSet.ONED) {
       for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
          GFPlane gframe = (GFPlane)e.nextElement();
          if (gframe.getID() == id) {
             frame1d_vec.removeElement(gframe);
             gframe = null;
             System.gc();

             svServer.controller.desktopRepaint();  
             return;
          }
       }
    } else if (dim == DataSet.TWOD) {
       for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
          GFrame2D gframe = (GFrame2D)e.nextElement();
          if (gframe.getID() == id) {
             frame2d_vec.removeElement(gframe);
             gframe = null;
             System.gc();

             svServer.controller.desktopRepaint();  
             return;
          }
       }
    } else if (dim == DataSet.THREED) {
       for (Enumeration e=frame3d_vec.elements(); e.hasMoreElements();) {
          GFrame3D gframe = (GFrame3D)e.nextElement();
          if (gframe.getID() == id) {
             frame3d_vec.removeElement(gframe);
             gframe = null;
             System.gc();

             svServer.controller.desktopRepaint();  
             return;
          }
       }
    } 
  }

  /**
   * return GFrame object from id
   * @param id GFrame id
   */
  public Object getGFrame(long id) {

     for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
        GFPlane gf1d = (GFPlane)e.nextElement();

        if (gf1d.getID() == id) {
           return gf1d;
        }
     }

     for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
        GFrame2D gf2d = (GFrame2D)e.nextElement();

        if (gf2d.getID() == id) {
           return gf2d;
        }
     }

     for (Enumeration e=frame3d_vec.elements(); e.hasMoreElements();) {
        GFrame3D gf3d = (GFrame3D)e.nextElement();

        if (gf3d.getID() == id) {
           return gf3d;
        }
     }

     return null;
  }



  /**
   * return 1d frames
   */
  public Vector get1dFrames() {
     return frame1d_vec;
  }

  /**
   * return 2d frames
   */
  public Vector get2dFrames() {
     return frame2d_vec;
  }

  /**
   * return 3d frames
   */
  public Vector get3dFrames() {
     return frame3d_vec;
  }

  /**
   * return ith GFPlane
   */
  public GFPlane get1dFrame(int i) {
     return (GFPlane)frame1d_vec.elementAt(i);
  }


  /**
   * return ith GFrame2D
   */
  public GFrame2D get2dFrame(int i) {
     return (GFrame2D)frame2d_vec.elementAt(i);
  }

  /**
   * return ith GFrame3D
   */
  public GFrame3D get3dFrame(int i) {
     return (GFrame3D)frame3d_vec.elementAt(i);
  }


  /**
   * return frame1d_vec size
   */
  public int get1dvecSize() {
     return frame1d_vec.size();
  }

  /**
   * return frame2d_vec size
   */
  public int get2dvecSize() {
     return frame2d_vec.size();
  }

  /**
   * return frame3d_vec size
   */
  public int get3dvecSize() {
     return frame3d_vec.size();
  }


  /**
   * kill all frames
   * @param b if true, broadcast
   */
  public void killAllFrames(boolean b) {
     if (b)
       svServer.cmanager.broadcastCommand(null, -1, null);

     while (!frame1d_vec.isEmpty()) {
        GFPlane gframe = (GFPlane)frame1d_vec.lastElement();
        gframe.killFrame();
     }

     while (!frame2d_vec.isEmpty()) {
        GFrame2D gframe = (GFrame2D)frame2d_vec.lastElement();
        gframe.killFrame();
     }

     while (!frame3d_vec.isEmpty()) {
        GFrame3D gframe = (GFrame3D)frame3d_vec.lastElement();
        gframe.killFrame();
     }

  }


  // received 1d data set
  private synchronized void received_1d(DataSet data, CPlane c,
                                             long id, boolean b) {
      boolean endProcess = false;
      String title = data.getTitle();
      
      for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
         GFPlane gframe = (GFPlane)e.nextElement();

         if (endProcess) break;
         if (id == gframe.getID()) {
            gframe.addDataSet((DSPlane)data);
            if (b)
              svServer.cmanager.broadcastDataSet(data, gframe.getID());
            return;
         }

         for (int i=0; i<gframe.dataSetVector.size(); i++) {
            DSPlane dataset = (DSPlane)gframe.dataSetVector.elementAt(i);

            if (title.equals(dataset.getTitle()) && 
     ((DSPlane)data).getDataType() == ((DSPlane)dataset).getDataType()) {
                for (int j=0; j<data.getnoOfTime(); j++) {
                    TDPlane td = ((DSPlane)data).timeData[j];
                    boolean done = false;

                    for (int k=0; k<dataset.getnoOfTime(); k++) {
		      if (td.getTime() == dataset.getTimeValue(k)) {
                        if (id == 0)
                           dataset.updateTimeData(td, k, true);
                        else
                           dataset.updateTimeData(td, k, false);
                        gframe.updateDataSet();
                        done = true;
                        break;
                      }
                    }

                    if (!done) {
                      dataset.addTimeData(td);
                      gframe.updateDataSet(i, dataset);
	            }
                }
                endProcess = true;
                if (b)
                  svServer.cmanager.broadcastDataSet(data, gframe.getID());

                break;
            } 
         }
      }

      if (!endProcess) {
         long frameID = register_1d(title, data, id, c);    
         if (b)
            svServer.cmanager.broadcastDataSet(data, frameID);
      }
  }

  private synchronized void received_2d(DataSet data, Config2D c,
                                             long id, boolean b) {     
      boolean endProcess = false;
      String title = data.getTitle();
      boolean done = false;

      for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
         GFrame2D gframe = (GFrame2D)e.nextElement();

         if (endProcess) break;
         if (id == gframe.getID()) {
            gframe.addDataSet((DataSet2D)data);
            if (b)
              svServer.cmanager.broadcastDataSet(data, gframe.getID());
            return;
         }

         for (int i=0; i<gframe.dataSetVector.size(); i++) {
            DataSet2D dataset = (DataSet2D)gframe.dataSetVector.elementAt(i);

            if (title.equals(dataset.getTitle())) {
                for (int j=0; j<data.getnoOfTime(); j++) {
                    TimeData2D td = ((DataSet2D)data).timeData[j];

                    for (int k=0; k<dataset.getnoOfTime(); k++) {
		      if (td.getTime() == dataset.getTimeValue(k)) {
                        dataset.updateTimeData(td, k);
                        gframe.updateDataSet();
                        done = true;
                        break;
                      }
                    }

                    if (!done) {
                      dataset.addTimeData(td);
                      gframe.updateDataSet(i, dataset);
	            }
                }
                endProcess = true;

                if (b)
                  svServer.cmanager.broadcastDataSet(data, gframe.getID());
                break;
            } 
         }
      }

      if (!endProcess) {
         long frameID = register_2d(title, data, id, c);    
         if (b)
            svServer.cmanager.broadcastDataSet(data, frameID);
      }
  }

  private synchronized void received_3d(DataSet data, Config3D c,
                                             long id, boolean b) {  

      boolean endProcess = false;
      String title = data.getTitle();
      boolean done = false;

      for (Enumeration e=frame3d_vec.elements(); e.hasMoreElements();) {
         GFrame3D gframe = (GFrame3D)e.nextElement();

         if (endProcess) break;
         for (int i=0; i<gframe.dataSetVector.size(); i++) {
            DataSet3D dataset = (DataSet3D)gframe.dataSetVector.elementAt(i);

            if (title.equals(dataset.getTitle())) {
                for (int j=0; j<data.getnoOfTime(); j++) {
                    TimeData3D td = ((DataSet3D)data).timeData[j];

                    for (int k=0; k<dataset.getnoOfTime(); k++) {
		      if (td.getTime() == dataset.getTimeValue(k)) {
                        dataset.updateTimeData(td, k);
                        gframe.updateDataSet();
                        done = true;
                        break;
                      }
                    }

                    if (!done) {
                      dataset.addTimeData(td);
                      gframe.updateDataSet(i, dataset, td);
	            }
                }
                endProcess = true;

                if (b)
                  svServer.cmanager.broadcastDataSet(data, gframe.getID());
                break;
            } 
         }
      }

      if (!endProcess) {
         long frameID = register_3d(title, data, id, c);    
         if (b)
            svServer.cmanager.broadcastDataSet(data, frameID);
      }
  }


  private synchronized long register_1d(String title, DataSet dataset, 
                                             long id, CPlane c) {
     if (c == null) {
        c = new CPlane();
        String[] str = svServer.configHandler.config1d.getSettings();
        c.doConfig(str);
        str[0] = svServer.configHandler.config1d.getColormap();
        c.setColorMap(str[0]);  
     }

     int type = ((DSPlane)dataset).getDataType();
   
     GFPlane gframe;
     if (type == DSPlane.NORMAL)
       gframe = new GFPOneD(title, (DataSet1D)dataset, c, svServer);
     else if (type == DSPlane.VECTOR)
       gframe = new GFPVec(title, (DataSetVec)dataset, c, svServer);  
     else if (type == DSPlane.CONNECTIVITY)
       gframe = new GFPConnect(title, (DataSetCon)dataset, c, svServer);  
     else if (type == DSPlane.BAR)
       gframe = new GFPBar(title, (DataSetBar)dataset, c, svServer);    
     else if (type == DSPlane.TREE)
       gframe = new GFPTree(title, (DataSetTree)dataset, c, svServer); 
     else
       return -1;

     frame1d_vec.addElement(gframe);
     if (id > 0)
        gframe.setID(id);

     gframe.setSize(400, 400);
     gframe.show();    
     gframe.setIconImage(SVUIC.iconImg);

     return gframe.getID();
  }

  private synchronized long register_2d(String title, DataSet dataset, 
                                              long id, Config2D c) {

     if (c == null) {
        c = new Config2D();
        String[] str = svServer.configHandler.config2d.getSettings();
        c.doConfig(str);
        str[0] = svServer.configHandler.config2d.getColormap();
        c.setColorMap(str[0]);
     }

     GFrame2D gframe = new GFrame2D(title, (DataSet2D)dataset, c, svServer);
     frame2d_vec.addElement(gframe);
     if (id > 0)
        gframe.setID(id);

     gframe.setSize(400, 400);
     gframe.show();    
     gframe.setIconImage(SVUIC.iconImg);

     return gframe.getID();
  }

  private synchronized long register_3d(String title, DataSet dataset,  
                                              long id, Config3D c) {

     if (c == null) {
        c = new Config3D();
        String[] str = svServer.configHandler.config3d.getSettings();
        c.doConfig(str);
        str[0] = svServer.configHandler.config3d.getColormap();
        c.setColorMap(str[0]);
     }

     GFrame3D gframe = new GFrame3D(title, (DataSet3D)dataset, c, svServer);
     frame3d_vec.addElement(gframe);
     if (id > 0)
        gframe.setID(id);

     gframe.setSize(400, 400);
     gframe.show();    
     gframe.setIconImage(SVUIC.iconImg);

     return gframe.getID();
  }

} // end of class







